###############################################################################
#
# (C) Copyright 2023 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################

###############################################################################
#   Imports
###############################################################################
from everysk.config import settings
from everysk.core.exceptions import SDKValueError
from everysk.core.fields import ListField, StrField, DictField

from everysk.sdk.entities.base import BaseEntity, ScriptMetaClass
from everysk.sdk.entities.fields import EntityNameField, EntityDescriptionField, EntityLinkUIDField, EntityWorkspaceField, EntityDateTimeField, EntityTagsField
from everysk.sdk.entities.query import Query
from everysk.sdk.entities.script import Script

###############################################################################
#   Report Implementation
###############################################################################
class Report(BaseEntity, metaclass=ScriptMetaClass):
    """
    This class represents a report entity object and provides methods to validate and manage the entity's data.

    Attributes:
        script (Script): The script object associated with the report.
        id (StrField): The unique identifier of the report.
        workspace (EntityWorkspaceField): The workspace of the report.
        name (EntityNameField): The name of the report.
        tags (EntityTagsField): The tags of the report.
        description (EntityDescriptionField): The description of the report.
        link_uid (EntityLinkUIDField): The link UID of the report.
        date (EntityDateTimeField): The date associated with the report.
        widgets (ListField): A list of widgets associated with the report.
        url (StrField): The URL of the report.
        authorization (StrField): The authorization string for accessing the report.
        config_cascaded (DictField): A dictionary of cascaded configuration settings.
        layout_content (DictField): A dictionary representing the layout content of the report.

        version (StrField): The version of the report.
        created_on (DateTimeField): The created on date of the report.
        updated_on (DateTimeField): The updated on date of the report.

    Example:
        >>> from everysk.sdk.entities.report.base import Report
        >>> report = Report()
        >>> report.script = 'my_script'
        >>> report.id = 'repo_12345678'
        >>> report.name = 'My Report'
        >>> report.tags = ['tag1', 'tag2']
        >>> report.description = 'This is a sample report.'
        >>> report.workspace = 'my_workspace'
        >>> report.date = '20220101'
        >>> report.widgets = [{'type': 'chart', 'data': {...}}, {'type': 'table', 'data': {...}}]
        >>> report.url = '/sefdsf5s54sdfsksdfs5'
        >>> report.authorization = 'private'
        >>> report.data_hash = 'hash123'
        >>> report.config_cascaded = {'setting1': 'value1', 'setting2': 'value2'}
        >>> report.layout_content = {'section1': {...}, 'section2': {...}}
        >>> report.create()
        >>> print(report)
        {
            'id': 'repo_12345678',
            'script': 'my_script',
            'name': 'My Report',
            'description': 'This is a sample report.',
            'tags': ['tag1', 'tag2'],
            'link_uid': None,
            'workspace': 'my_workspace',
            'date': '20220101',
            'widgets': [{'type': 'chart', 'data': {...}}, {'type': 'table', 'data': {...}}],
            'url': '/sefdsf5s54sdfsksdfs5',
            'authorization': 'private',
            'config_cascaded': {'setting1': 'value1', 'setting2': 'value2'},
            'layout_content': {'section1': {...}, 'section2': {...}}
        }
    """

    script: Script = None
    _orderable_attributes = ListField(default=['date', 'created_on', 'updated_on', 'name'])

    id = StrField(regex=settings.REPORT_ID_REGEX, max_size=settings.REPORT_ID_MAX_SIZE, required_lazy=True, empty_is_none=True)

    name = EntityNameField()
    description = EntityDescriptionField()
    tags = EntityTagsField()
    link_uid = EntityLinkUIDField()
    workspace = EntityWorkspaceField()
    date = EntityDateTimeField()

    widgets = ListField()
    url = StrField(required_lazy=True)
    authorization = StrField(default=None, regex=settings.REPORT_AUTHORIZATION_REGEX, required_lazy=True)
    config_cascaded = DictField(required_lazy=True)
    layout_content = DictField(required_lazy=True)

    def validate(self) -> bool:
        """
        This method validates the entity object and raises an exception if it is not
        valid. The validation is performed by calling the `validate` method of each field
        of the entity.

        Args:
            self (Self): The entity object to be validated.

        Raises:
            FieldValueError: If the entity object is not valid.

        Returns:
            bool: True if the entity object is valid, False otherwise.

        Example:
            >>> entity = Entity()
            >>> entity.validate()
            True
        """
        return self.get_response(self_obj=self)

    @staticmethod
    def get_id_prefix() -> str:
        """
        Returns the prefix of the report id field value.

        Returns:
            str: The prefix of the report id field value.

        Example:
            >>> from everysk.sdk.entities.report.base import Report
            >>> Report.get_id_prefix()
            'repo_'

        Notes:
            The prefix is typically used to distinguish report IDs from the other types of IDs
        """
        return settings.REPORT_ID_PREFIX

    def _check_query(self, query: Query) -> bool:
        """
        Check the query object.

        Args:
            query (Query): The query object.

        Returns:
            Query: The query object.
        """
        return True

    def _check_entity_to_query(self) -> bool:
        """
        Check the entity object to query.

        Returns:
            bool: True if the entity object is valid.
        """
        super()._check_entity_to_query()
        if self.url and (self.name or self.tags or self.link_uid):
            raise SDKValueError("Can't filter by URL and Name, Tags or Link UID at the same time")

        return True

    def _mount_query(self, query: Query) -> Query:
        """
        Mount the query object.

        Args:
            query (Query): The query object.

        Returns:
            Query: The query object.
        """
        query = super()._mount_query(query)
        if self.url is not None:
            query = query.where('url', self.url)

        return query

    def to_dict(self, add_class_path: bool = False, recursion: bool = False) -> dict:
        """
        Convert the entity to a JSON-serializable dictionary.
        This method converts the entity object into a dictionary that can be easily
        serialized to JSON.

        Args:
            self (Self): The entity instance to convert.
            with_internals (bool, optional): Whether to include internal parameters. Defaults to True.
            recursion (bool, optional): Whether to include nested entities. Defaults to False.

        Returns:
            dict: A dictionary representation of the entity.

        Raises:
            NotImplementedError: This method should be implemented in subclasses.
        """
        dct: dict = super().to_dict(add_class_path=add_class_path, recursion=recursion)

        if add_class_path is False:
            relative_url: None | str = None
            absolute_url: None | str = None
            if self.url is not None:
                relative_url = f'{settings.REPORT_URL_PATH}{self.url}'
                absolute_url = f'{settings.EVERYSK_APP_URL}{relative_url}'

            dct['url'] = absolute_url
            dct['absolute_url'] = absolute_url
            dct['relative_url'] = relative_url

            dct['authorization'] = dct['authorization'].upper() if dct['authorization'] else None

            dct.pop('config_cascaded', None)
            dct.pop('layout_content', None)

        return dct



###############################################################################
#
# (C) Copyright 2023 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################

###############################################################################
#   Imports
###############################################################################
from everysk.config import settings
from everysk.core.compress import compress
from everysk.core.datetime import DateTime
from everysk.core.exceptions import FieldValueError, SDKValueError
from everysk.core.http import HttpSDKPOSTConnection, httpx
from everysk.core.unittests import TestCase, mock

from everysk.sdk.entities.custom_index.base import CustomIndex


###############################################################################
#   Custom Index TestCase Implementation
###############################################################################
class CustomIndexTestCase(TestCase):

    def setUp(self):
        self.sample_data = {
            'symbol': 'CUSTOM:INDEX',
            'name': 'SampleCustomIndex',
            'tags': ['tag1', 'tag2'],
            'description': 'Description',
            'currency': 'USD',
            'data': [[1, 2, 3], [4, 5, 6]],
            'periodicity': 'M',
            'data_type': 'PRICE',
            'base_price': 100,
            'version': '1.0',
            'created_on': DateTime(2023, 9, 9, 9, 9, 9, 9),
            'updated_on': DateTime(2023, 9, 9, 9, 9, 9, 9),
        }
        self.custom_index = CustomIndex(**self.sample_data)
        self.headers = HttpSDKPOSTConnection().get_headers()
        self.api_url = HttpSDKPOSTConnection().get_url()

    def test_init(self):
        self.assertEqual(self.custom_index.symbol, self.custom_index.id)
        self.assertEqual(self.custom_index.symbol, self.sample_data['symbol'])
        self.assertEqual(self.custom_index.name, self.sample_data['name'])
        self.assertEqual(self.custom_index.tags, self.sample_data['tags'])
        self.assertEqual(self.custom_index.description, self.sample_data['description'])
        self.assertEqual(self.custom_index.currency, self.sample_data['currency'])
        self.assertEqual(self.custom_index.data, self.sample_data['data'])
        self.assertEqual(self.custom_index.periodicity, self.sample_data['periodicity'])
        self.assertEqual(self.custom_index.data_type, self.sample_data['data_type'])
        self.assertEqual(self.custom_index.base_price, self.sample_data['base_price'])
        self.assertEqual(self.custom_index.version, self.sample_data['version'])
        self.assertEqual(self.custom_index.created_on, self.sample_data['created_on'])
        self.assertEqual(self.custom_index.updated_on, self.sample_data['updated_on'])

    def test_property_id(self):
        self.assertEqual(self.custom_index.id, self.custom_index.symbol)

    def test_setter_id(self):
        self.custom_index.id = 'CUSTOM:NEW_ID'
        self.assertEqual(self.custom_index.id, 'CUSTOM:NEW_ID')
        self.assertEqual(self.custom_index.symbol, 'CUSTOM:NEW_ID')
        self.custom_index.symbol = 'CUSTOM:NEW_ID_2'
        self.assertEqual(self.custom_index.id, 'CUSTOM:NEW_ID_2')
        self.assertEqual(self.custom_index.symbol, 'CUSTOM:NEW_ID_2')

    def test_setter_id_with_invalid_value(self):
        with self.assertRaises(FieldValueError) as e:
            self.custom_index.id = 'Potato'
        self.assertEqual("The value 'Potato' for field 'symbol' must match with this regex: ^CUSTOM:[A-Z0-9_]*$.", e.exception.msg)

        with self.assertRaises(FieldValueError) as e:
            self.custom_index.symbol = 'Potato'
        self.assertEqual("The value 'Potato' for field 'symbol' must match with this regex: ^CUSTOM:[A-Z0-9_]*$.", e.exception.msg)

    def test_get_id_prefix(self):
        self.assertEqual(CustomIndex.get_id_prefix(), settings.CUSTOM_INDEX_SYMBOL_PREFIX)

    def test_validate(self):
        expected_data = compress({'class_name': 'CustomIndex', 'method_name': 'validate', 'self_obj': self.custom_index.to_dict(add_class_path=True), 'params': {}}, protocol='gzip', serialize='json')
        custom_index: CustomIndex = self.custom_index

        with mock.patch('httpx.Client.post') as mock_post:
            mock_post.return_value.content = '{}'
            mock_post.return_value.status_code = 200
            custom_index.validate()

        mock_post.assert_called_with(
            url=self.api_url,
            headers=self.headers,
            timeout=httpx.Timeout(timeout=30, read=settings.EVERYSK_SDK_HTTP_DEFAULT_TIMEOUT),
            data=expected_data
        )

    def test_validate_transient(self):
        expected_data = compress({'class_name': 'CustomIndex', 'method_name': 'validate', 'self_obj': self.custom_index.to_dict(add_class_path=True), "params": {}}, protocol='gzip', serialize='json')
        cmp_entity = self.custom_index.copy()
        cmp_entity.update(
            created_on=None,
            updated_on=None,
        )

        with mock.patch('httpx.Client.post') as mock_post, \
            mock.patch('everysk.core.datetime.DateTime.now') as mock_datetime_now:

            mock_datetime_now.return_value = DateTime(2023, 9, 9, 9, 9, 9, 9)
            mock_post.return_value.content = '{}'
            mock_post.return_value.status_code = 200
            CustomIndex.validate_transient(self.custom_index)

        mock_post.assert_called_with(
            url=self.api_url,
            headers=self.headers,
            timeout=httpx.Timeout(timeout=30, read=settings.EVERYSK_SDK_HTTP_DEFAULT_TIMEOUT),
            data=expected_data
        )

    def test_entity_to_query(self):
        custom_index: CustomIndex = self.custom_index.copy()
        self.assertRaisesRegex(
            SDKValueError,
            "Can't filter by Name and Tags at the same time",
            custom_index.to_query
        )

        # test with name
        custom_index.pop('name')
        query = custom_index.to_query() #pylint: disable=protected-access
        self.assertListEqual(query.filters, [('tags', '=', 'tag1'), ('tags', '=', 'tag2')])
        self.assertEqual(query.order, [])
        self.assertEqual(query.projection, None)
        self.assertEqual(query.limit, None)
        self.assertEqual(query.offset, None)
        self.assertEqual(query.page_size, None)
        self.assertEqual(query.page_token, None)
        self.assertEqual(query.distinct_on, [])

        # test with tags
        custom_index: CustomIndex = self.custom_index.copy()
        custom_index.pop('tags')
        query = custom_index.to_query() #pylint: disable=protected-access
        self.assertListEqual(query.filters, [('name', '<', 'samplecustomindey'), ('name', '>=', 'samplecustomindex')])
        self.assertEqual(query.order, [])
        self.assertEqual(query.projection, None)
        self.assertEqual(query.limit, None)
        self.assertEqual(query.offset, None)
        self.assertEqual(query.page_size, None)
        self.assertEqual(query.page_token, None)
        self.assertEqual(query.distinct_on, [])

    def test_modify_many(self):
        with self.assertRaises(NotImplementedError):
            CustomIndex.modify_many(entity_id_list=['CUSTOM:INDEX'], overwrites={})

    def test_clone_many(self):
        with self.assertRaises(NotImplementedError):
            CustomIndex.clone_many(entity_id_list=['CUSTOM:INDEX'], overwrites={})



###############################################################################
#
# (C) Copyright 2023 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
# pylint: disable=protected-access
from google.api_core.exceptions import AlreadyExists, DeadlineExceeded, ServiceUnavailable, TooManyRequests
from everysk.core.compress import compress
from everysk.core.exceptions import FieldValueError, RequiredError
from everysk.core import workers
from everysk.core.workers import (
    BaseGoogle, CloudTasksClient, CreateTaskRequest, HttpMethod, log as worker_log,
    PauseQueueRequest, ResumeQueueRequest, TaskGoogle, WorkerGoogle
)
from everysk.core.unittests import TestCase, mock


class BaseGoogleTestCase(TestCase):

    def test_required_attributes(self):
        self.assertRaisesRegex(
            RequiredError,
            'The worker_id attribute is required.',
            BaseGoogle,
            google_task_project='bla',
            google_task_location='bla'
        )

    @mock.patch.object(workers, 'CloudTasksClient', spec=CloudTasksClient)
    def test_gtask(self, CloudTasksClient: mock.MagicMock): # pylint: disable=redefined-outer-name
        gtask = BaseGoogle(google_task_project='bla', google_task_location='bla', worker_id='bla')
        self.assertEqual(gtask._gtask, BaseGoogle._gtask)
        self.assertEqual(gtask.gtask, gtask._gtask)
        self.assertEqual(gtask.gtask, CloudTasksClient.return_value)


class FakeTask(TaskGoogle):
    google_task_location = 'location'
    google_task_project = 'project'
    worker_id = 'worker_id'
    worker_url = 'worker_url'

    def run(self) -> str:
        return super().run() or 'run'


@mock.patch.object(BaseGoogle, 'gtask', spec=CloudTasksClient)
class TaskGoogleTestCase(TestCase):

    def setUp(self) -> None:
        self.task = FakeTask()
        self.request = CreateTaskRequest(parent='queue_path', task={
            'name': 'task_path',
            'dispatch_deadline': {'seconds': 600},
            'http_request': {
                'http_method': HttpMethod.POST,
                'url': f'{self.task.worker_url}/worker/{self.task.worker_id}',
                'headers': self.task.get_headers(),
                'body': compress({
                    'cls': self.task.get_full_doted_class_path(),
                    'kwargs': self.task._received_kwargs
                }, serialize='pickle')
            }
        })

    def test_init(self, gtask: mock.MagicMock):
        task01 = FakeTask()
        task02 = FakeTask()
        task03 = FakeTask(google_task_name='test')
        self.assertNotEqual(task01.google_task_name, task02.google_task_name)
        self.assertNotEqual(task01.google_task_name, task03.google_task_name)
        self.assertNotEqual(task02.google_task_name, task03.google_task_name)
        self.assertEqual(task03.google_task_name, 'test')
        gtask.assert_not_called()

    def test_gtask_queue_path(self, gtask: mock.MagicMock):
        result = self.task.gtask_queue_path()
        gtask.queue_path.assert_called_once_with(project='project', location='location', queue='worker_id')
        self.assertEqual(result, gtask.queue_path.return_value)

    def test_gtask_task_path(self, gtask: mock.MagicMock):
        result = self.task.gtask_task_path()
        gtask.task_path.assert_called_once_with(
            project='project', location='location', queue='worker_id', task=self.task.google_task_name
        )
        self.assertEqual(result, gtask.task_path.return_value)

    def test_gtask_create_task_request(self, gtask: mock.MagicMock):
        gtask.task_path.return_value = 'task_path'
        gtask.queue_path.return_value = 'queue_path'
        result = self.task.gtask_create_task_request({
            'cls': self.task.get_full_doted_class_path(),
            'kwargs': self.task._received_kwargs
        })
        self.assertEqual(result, self.request)

    def test_get_headers(self, gtask: mock.MagicMock):
        self.assertEqual(self.task.get_headers(), {'Content-type': 'application/octet-stream'})
        gtask.assert_not_called()

    def test_run(self, gtask: mock.MagicMock):
        self.assertEqual(self.task.run(), 'run')
        gtask.assert_not_called()

    def test_send_start(self, gtask: mock.MagicMock):
        gtask.queue_path.return_value = 'queue_path'
        self.task.send_start()
        resume_queue_request = ResumeQueueRequest(name=self.task.gtask_queue_path())
        gtask.resume_queue.assert_called_once_with(request=resume_queue_request)

    def test_send_pause(self, gtask: mock.MagicMock):
        gtask.queue_path.return_value = 'queue_path'
        self.task.send_pause()
        pause_queue_request = PauseQueueRequest(name=self.task.gtask_queue_path())
        gtask.pause_queue.assert_called_once_with(request=pause_queue_request)

    def test_save(self, gtask: mock.MagicMock):
        gtask.task_path.return_value = 'task_path'
        gtask.queue_path.return_value = 'queue_path'
        self.task.save()
        gtask.create_task.assert_called_once_with(request=self.request, timeout=30)

    def test_save_timeout(self, gtask: mock.MagicMock):
        gtask.task_path.return_value = 'task_path'
        gtask.queue_path.return_value = 'queue_path'
        self.task.save(timeout=15)
        gtask.create_task.assert_called_once_with(request=self.request, timeout=15)

    def test_save_timeout_greater_30(self, gtask: mock.MagicMock):
        gtask.task_path.return_value = 'task_path'
        gtask.queue_path.return_value = 'queue_path'
        self.task.save(timeout=60)
        gtask.create_task.assert_called_once_with(request=self.request, timeout=30)

    @mock.patch.object(workers.log, 'debug')
    def test_save_retry_already_error(self, debug: mock.MagicMock, gtask: mock.MagicMock):
        gtask.task_path.return_value = 'task_path'
        gtask.queue_path.return_value = 'queue_path'
        gtask.create_task.side_effect = AlreadyExists('Error')
        call = mock.call(request=self.request, timeout=30)
        self.task.save(retry_times=1)
        gtask.create_task.assert_has_calls([call])
        debug.assert_called_once_with('Google task already exists: %s', 'task_path')

    def test_save_retry_deadline_error(self, gtask: mock.MagicMock):
        gtask.task_path.return_value = 'task_path'
        gtask.queue_path.return_value = 'queue_path'
        gtask.create_task.side_effect = DeadlineExceeded('Error')
        call = mock.call(request=self.request, timeout=30)
        with self.assertRaisesRegex(DeadlineExceeded, '504 Error'):
            self.task.save(retry_times=1)
        gtask.create_task.assert_has_calls([call, call])

    def test_save_retry_service_error(self, gtask: mock.MagicMock):
        gtask.task_path.return_value = 'task_path'
        gtask.queue_path.return_value = 'queue_path'
        gtask.create_task.side_effect = ServiceUnavailable('Error')
        call = mock.call(request=self.request, timeout=30)
        with self.assertRaisesRegex(ServiceUnavailable, '503 Error'):
            self.task.save(retry_times=1)
        gtask.create_task.assert_has_calls([call, call])

    def test_save_retry_429_error(self, gtask: mock.MagicMock):
        gtask.task_path.return_value = 'task_path'
        gtask.queue_path.return_value = 'queue_path'
        gtask.create_task.side_effect = TooManyRequests('Error')
        call = mock.call(request=self.request, timeout=30)
        with self.assertRaisesRegex(TooManyRequests, '429 Error'):
            self.task.save(retry_times=1)
        gtask.create_task.assert_has_calls([call, call])

    def test_deadline_min_value(self, gtask: mock.MagicMock):
        self.assertRaisesRegex(
            FieldValueError,
            "The value '14' for field 'timeout' must be between 15 and 1800.",
            FakeTask,
            timeout=14
        )
        gtask.assert_not_called()

    def test_deadline_max_value(self, gtask: mock.MagicMock):
        self.assertRaisesRegex(
            FieldValueError,
            "The value '1801' for field 'timeout' must be between 15 and 1800.",
            FakeTask,
            timeout=1801
        )
        gtask.assert_not_called()


class FakeWorker(WorkerGoogle):
    google_task_location = 'location'
    google_task_project = 'project'
    worker_id = 'worker_id'


class WorkerGoogleTestCase(TestCase):

    def test_check_google_task(self):
        worker = FakeWorker()
        headers = {'X-Cloudtasks-Queuename': 'worker_id', 'User-Agent': 'Google-Cloud-Tasks'}
        self.assertTrue(worker.check_google_task(headers))
        headers = {'X-Cloudtasks-Queuename': 'worker', 'User-Agent': 'Google-Cloud-Tasks'}
        self.assertFalse(worker.check_google_task(headers))
        headers = {'X-Cloudtasks-Queuename': 'worker_id', 'User-Agent': 'Google-Cloud'}
        self.assertFalse(worker.check_google_task(headers))

    @mock.patch.object(FakeTask, 'run', return_value='run_task')
    def test_run_task(self, run: mock.MagicMock):
        worker = FakeWorker()
        self.assertEqual(worker.run_task(FakeTask().get_full_doted_class_path(), {}), 'run_task')
        run.assert_called_once_with()

    @mock.patch.object(FakeTask, 'run', return_value='worker_run')
    def test_worker_run(self, run: mock.MagicMock):
        headers = {'X-Cloudtasks-Queuename': 'worker_id', 'User-Agent': 'Google-Cloud-Tasks'}
        data = compress({'cls': FakeTask().get_full_doted_class_path(), 'kwargs': {}}, serialize='pickle')
        self.assertDictEqual(
            FakeWorker.worker_run(headers, data),
            {'error': False, 'result': 'worker_run'}
        )
        run.assert_called_once_with()

    @mock.patch.object(FakeTask, 'run', side_effect=ValueError('Error'))
    @mock.patch.object(worker_log, 'error')
    @mock.patch.object(workers.traceback, 'format_exc', return_value='Traceback Error')
    def test_worker_run_error(self, format_exc: mock.MagicMock, log: mock.MagicMock, run: mock.MagicMock):
        headers = {'X-Cloudtasks-Queuename': 'worker_id', 'User-Agent': 'Google-Cloud-Tasks'}
        data = compress({'cls': FakeTask().get_full_doted_class_path(), 'kwargs': {}}, serialize='pickle')
        self.assertDictEqual(
            FakeWorker.worker_run(headers, data),
            {'error': True, 'message': 'Worker worker_id error: Error'}
        )
        run.assert_called_once_with()
        log.assert_called_once_with('Worker %s error: %s', 'worker_id', 'Traceback Error')
        format_exc.assert_called_once_with()

    @mock.patch.object(FakeTask, 'run', return_value='worker_run')
    @mock.patch.object(worker_log, 'error')
    def test_worker_run_check_fail(self, log: mock.MagicMock, run: mock.MagicMock):
        headers = {'X-Cloudtasks-Queuename': 'id', 'User-Agent': 'Google-Cloud-Tasks'}
        data = compress({'cls': FakeTask().get_full_doted_class_path(), 'kwargs': {}}, serialize='pickle')
        self.assertDictEqual(
            FakeWorker.worker_run(headers, data),
            {
                'error': True,
                'message': "Couldn't validate Google headers - {'X-Cloudtasks-Queuename': 'id', 'User-Agent': 'Google-Cloud-Tasks'}"
            }
        )
        run.assert_not_called()
        log.assert_called_once_with(
            "Couldn't validate Google headers - {'X-Cloudtasks-Queuename': 'id', 'User-Agent': 'Google-Cloud-Tasks'}"
        )

    @mock.patch.object(FakeTask, 'run', return_value='worker_run')
    @mock.patch.object(worker_log, 'error')
    def test_worker_run_worker_id(self, log: mock.MagicMock, run: mock.MagicMock):
        headers = {'X-Cloudtasks-Queuename': 'worker_id', 'User-Agent': 'Google-Cloud-Tasks'}
        data = compress({'cls': FakeTask().get_full_doted_class_path(), 'kwargs': {}}, serialize='pickle')
        self.assertDictEqual(
            FakeWorker.worker_run(headers, data, worker_id='new_id'),
            {
                'error': True,
                'message': "Couldn't validate Google headers - {'X-Cloudtasks-Queuename': 'worker_id', 'User-Agent': 'Google-Cloud-Tasks'}"
            }
        )
        run.assert_not_called()
        log.assert_called_once_with("Couldn't validate Google headers - {'X-Cloudtasks-Queuename': 'worker_id', 'User-Agent': 'Google-Cloud-Tasks'}")
